/*
 *  anytun
 *
 *  The secure anycast tunneling protocol (satp) defines a protocol used
 *  for communication between any combination of unicast and anycast
 *  tunnel endpoints.  It has less protocol overhead than IPSec in Tunnel
 *  mode and allows tunneling of every ETHER TYPE protocol (e.g.
 *  ethernet, ip, arp ...). satp directly includes cryptography and
 *  message authentication based on the methodes used by SRTP.  It is
 *  intended to deliver a generic, scaleable and secure solution for
 *  tunneling and relaying of packets of any protocol.
 *
 *
 *  Copyright (C) 2007-2008 Othmar Gsenger, Erwin Nindl, 
 *                          Christian Pointner <satp@wirdorange.org>
 *
 *  This file is part of Anytun.
 *
 *  Anytun is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 3 as
 *  published by the Free Software Foundation.
 *
 *  Anytun is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with anytun.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _RTPSESSION_H_
#define _RTPSESSION_H_

#include "threadUtils.hpp"

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>

class RtpSession
{
public:
	RtpSession(const std::string& call_id);

  bool isDead();
  bool isDead(bool d);

  bool isComplete();
  bool isComplete(bool c);

  std::string getLocalAddr();
  RtpSession& setLocalAddr(std::string a);
  u_int16_t getLocalPort1();
  RtpSession& setLocalPort1(u_int16_t p);
  u_int16_t getLocalPort2();
  RtpSession& setLocalPort2(u_int16_t p);


  u_int16_t getRemotePort1();
  RtpSession& setRemotePort1(u_int16_t p);
  std::string getRemoteAddr1();
  RtpSession& setRemoteAddr1(std::string a);

  u_int16_t getRemotePort2();
  RtpSession& setRemotePort2(u_int16_t p);
  std::string getRemoteAddr2();
  RtpSession& setRemoteAddr2(std::string a);

	RtpSession& setSeen1();
  bool getSeen1();

	RtpSession& setSeen2();
  bool getSeen2();

private:
	RtpSession(const RtpSession & src);
  
  void reinit();

  //TODO: check if this is ok
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version)
	{    
    Lock lock(mutex_);

    ar & dead_;
    ar & complete_;
    ar & local_addr_;
    ar & local_port1_;
    ar & local_port2_;
    ar & remote_addr1_;
    ar & remote_port1_;
    ar & remote_addr2_;
    ar & remote_port2_;
		ar & seen1_;
		ar & seen2_;

    if(complete_ && !dead_)
      reinit();

    in_sync_ = true;
	}

  bool in_sync_;
	::Mutex mutex_;

  const std::string& call_id_;
  bool dead_;
  bool complete_;
  std::string local_addr_;
  u_int16_t local_port1_, local_port2_;
  std::string remote_addr1_, remote_addr2_;
  u_int16_t remote_port1_, remote_port2_;
	bool seen1_,seen2_; //has at least 1 packet been recieved?
};


#endif
